@@ -1,5 +1,5 @@
 /*
- * /dev/3dfx device for 2.x kernels with MTRR settings enabled.
+ * /dev/3dfx device for Linux kernels with MTRR settings enabled.
  *
  * Original by Daryll Straus.
  * Port to 2.1 kernel by Jon Taylor.
@@ -27,13 +27,18 @@
 /*
  * Include this first as it defines things that affect the kernel headers.
  */
-#include <linux/autoconf.h>
 #include <linux/version.h>
 
 #ifndef KERNEL_VERSION
 #define KERNEL_VERSION(a,b,c) (((a) << 16) + ((b) << 8) + (c))
 #endif
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,33)
+#include <generated/autoconf.h>
+#else
+#include <linux/autoconf.h>
+#endif
+
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,1,115)
 /*
  * It might work with smaller kernels, but I never tested that.
@@ -636,9 +641,18 @@ static int doPIO(unsigned int cmd, unsigned long arg)
 	return -EINVAL;
 }
 
+#ifdef HAVE_UNLOCKED_IOCTL
+static long ioctl_3dfx(struct file *file, unsigned int cmd, unsigned long arg)
+#else
 static int ioctl_3dfx(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
+#endif
 {
+#ifdef HAVE_UNLOCKED_IOCTL
+	DEBUGMSG(("3dfx: Entering ioctl_3dfx, file %p cmd %x arg %lx\n",
+	          file, cmd, arg));
+#else
 	DEBUGMSG(("3dfx: Entering ioctl_3dfx, inode %p file %p cmd %x arg %lx\n", inode, file, cmd, arg));
+#endif
 
 	switch (_IOC_TYPE(cmd)) {
 	case '3':
@@ -772,7 +786,11 @@ static struct file_operations fops_3dfx = {
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
 	.owner =	THIS_MODULE,
 #endif
+#ifdef HAVE_UNLOCKED_IOCTL
+	.unlocked_ioctl = ioctl_3dfx,
+#else
 	.ioctl =	ioctl_3dfx,
+#endif
 	.mmap =		mmap_3dfx,
 	.open =		open_3dfx,
 	.release =	release_3dfx,
@@ -908,7 +926,7 @@ MODULE_AUTHOR("Daryll Strauss et al.");
 MODULE_DESCRIPTION("3dfx glide kernel device driver");
 MODULE_LICENSE("GPL");
 #ifdef MODULE_VERSION
-MODULE_VERSION("2008.06.24");
+MODULE_VERSION("2011.05.27");
 #endif
 #endif
 
